@@ -1,9 +1,9 @@
 package Params::Sah;
 
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
-our $DATE = '2020-05-08'; # DATE
+our $DATE = '2020-05-10'; # DATE
 our $DIST = 'Params-Sah'; # DIST
-our $VERSION = '0.071'; # VERSION
+our $VERSION = '0.072'; # VERSION
 
 use 5.010001;
 use strict;
@@ -13,6 +13,7 @@ use Log::ger;
 use Exporter qw(import);
 our @EXPORT_OK = qw(gen_validator);
 
+our $OPT_BACKEND        = 'Data::Sah';
 our $OPT_ON_INVALID     = 'croak';
 our $OPT_INVALID_DETAIL = 0;
 our $OPT_NAMED          = 0;
@@ -20,8 +21,16 @@ our $OPT_DISABLE        = 0;
 our $OPT_ALLOW_EXTRA    = 0;
 our $DEBUG;
 
+sub _plc {
+    require Data::Sah;
+    state $sah = Data::Sah->new;
+    state $plc = $sah->get_compiler('perl');
+    $plc;
+}
+
 sub gen_validator {
-    my ($opt_on_invalid, $opt_named, $opt_disable, $opt_allow_extra,
+    my ($opt_backend,
+        $opt_on_invalid, $opt_named, $opt_disable, $opt_allow_extra,
         $opt_invalid_detail, $opt_optional_params);
     {
         my $opts;
@@ -30,12 +39,18 @@ sub gen_validator {
         } else {
             $opts = {};
         }
+
+        $opt_backend = delete $opts->{backend} // $OPT_BACKEND // 'Data::Sah';
+        die "Invalid backend value, must be: Data::Sah|Data::Sah::Tiny"
+            unless $opt_backend =~ /\A(Data::Sah::Tiny|Data::Sah)\z/;
         $opt_on_invalid = delete $opts->{on_invalid} // $OPT_ON_INVALID //
             'croak';
         die "Invalid on_invalid value, must be: croak|carp|warn|die|return"
             unless $opt_on_invalid =~ /\A(croak|carp|warn|die|return)\z/;
 
         $opt_invalid_detail = delete $opts->{invalid_detail} // $OPT_INVALID_DETAIL // 0;
+        die "Data::Sah::Tiny does not support invalid_detail=>1"
+            if $opt_backend eq 'Data::Sah::Tiny' && $opt_invalid_detail;
         $opt_named = delete $opts->{named} // $OPT_NAMED // 0;
         $opt_disable = delete $opts->{disable} // $OPT_DISABLE // 0;
         $opt_allow_extra = delete $opts->{allow_extra} // $OPT_ALLOW_EXTRA // 0;
@@ -47,10 +62,6 @@ sub gen_validator {
         return $opt_on_invalid eq 'str' ? sub {''} : sub {1};
     }
 
-    require Data::Sah;
-    state $sah = Data::Sah->new;
-    state $plc = $sah->get_compiler('perl');
-
     require Carp;
     require Data::Dmp;
 
@@ -135,14 +146,23 @@ sub gen_validator {
             $argterm = '$_ps_args->['.$argname.']';
             $data_name = "arg$argname";
         }
-        my $cd = $plc->compile(
-            data_name    => $data_name,
-            data_term    => $argterm,
-            err_term     => '$_ps_res',
-            schema       => $schemas{$argname},
-            return_type  => $opt_invalid_detail ? 'str' : 'bool',
-            indent_level => 1,
-        );
+        my $cd;
+        if ($opt_backend eq 'Data::Sah') {
+            $cd = _plc->compile(
+                data_name    => $data_name,
+                data_term    => $argterm,
+                err_term     => '$_ps_res',
+                schema       => $schemas{$argname},
+                return_type  => $opt_invalid_detail ? 'str' : 'bool',
+                indent_level => 1,
+            );
+        } else {
+            require Data::Sah::Tiny;
+            $cd = Data::Sah::Tiny::gen_validator($schemas{$argname}, {
+                hash => 1,
+                data_term => $argterm,
+            });
+        }
         die "Incompatible Data::Sah version (cd v=$cd->{v}, expected 2)" unless $cd->{v} == 2;
         for my $mod_rec (@{ $cd->{modules} }) {
             next unless $mod_rec->{phase} eq 'runtime';
@@ -196,7 +216,7 @@ Params::Sah - Validate method/function parameters using Sah schemas
 
 =head1 VERSION
 
-This document describes version 0.071 of Params::Sah (from Perl distribution Params-Sah), released on 2020-05-08.
+This document describes version 0.072 of Params::Sah (from Perl distribution Params-Sah), released on 2020-05-10.
 
 =head1 SYNOPSIS
 
@@ -296,6 +316,10 @@ L<Sah> schemas.
 
 Bool. If set to true will print validator code when generated.
 
+=head2 $OPT_BACKEND
+
+Str. Used to set default for C<backend> option.
+
 =head2 $OPT_ALLOW_EXTRA
 
 Bool. Used to set default for C<allow_extra> option.
@@ -341,6 +365,11 @@ Known options:
 
 =over
 
+=item * backend => str (default: Data::Sah)
+
+Can be set to the experimental L<Data::Sah::Tiny> to speed up validator
+generation for simpler schemas.
+
 =item * named => bool (default: 0)
 
 If set to true, it means we are generating validator for subroutine that accepts
